'use strict';

var _require = require('chai'),
    expect = _require.expect;

var opsSchema = require('../../lib/schemas/ops');
var schemaValidator = require('../../lib/schemas/validator');

var validate = schemaValidator(opsSchema);

describe('validator (ops)', function () {
  it('should allow a valid schema', function () {
    var ops = [{
      method: 'method1',
      args: {}
    }];
    var validated = validate(ops);

    expect(validated).to.deep.equal(ops);
  });

  it('should throw if method is not specified', function () {
    var invalidOps = [{
      args: {}
    }];
    expect(function () {
      return validate(invalidOps);
    }).to.throw('required');
  });

  it('should throw if method is not a string', function () {
    var invalidOps = [{
      method: 1,
      args: {}
    }];
    expect(function () {
      return validate(invalidOps);
    }).to.throw('string');
  });

  it('should throw if method is empty', function () {
    var invalidOps = [{
      method: '',
      args: {}
    }];
    expect(function () {
      return validate(invalidOps);
    }).to.throw('empty');
  });

  it('should throw if args is not specified', function () {
    var invalidOps = [{
      method: 'method1'
    }];
    expect(function () {
      return validate(invalidOps);
    }).to.throw('required');
  });

  it('should throw if args is not an object', function () {
    var invalidOps = [{
      method: 'method1',
      args: 10
    }];
    expect(function () {
      return validate(invalidOps);
    }).to.throw('object');
  });
});